import socket
import struct
import threading
import time
from typing import Callable

from prontonet.protocol import ProntonetProtocol
from prontonet.structures import ProntonetCommand, StatusLineStatusChanged, StatusAlarmsStatusChanged, \
    StatusDecoderAudioModeChanged, StatusEncoderAudioModeChanged
from prontonet.enums import Command, AlarmStatus, LineStatus, IPCallType, Codec


class ProntonetDevice:
    """
    A class used to represent Prontonet Device.

    ...

    Attributes
    ----------
    ip : str
        IP address of Prontonet device.
    port : int
        Port number of command protocol.
    status_port : int
        Port number of status protocol.

    Methods
    -------
    connect()
        Connects to status protocol socket and starts listening for messages.

    disconnect()
        Disconnects from status protocol socket.

    attach_on_connected(func, *args)
        Method to attach function, which will be executed after status socket connected.

    attach_on_status_socket_disconnected(func, *args)
        Method to attach function, which will be executed after status socket disconnected.

    attach_on_line_status_changed(func, *args)
        Method to attach function, which will be executed after line status changed.

    attach_on_alarm_status_changed(func, *args)
        Method to attach function, which will be executed after alarm status changed.

    attach_on_decoder_audio_mode_changed(func, *args)
        Method to attach function, which will be executed after decoder audio mode changed.

    attach_on_encoder_audio_mode_changed(func, *args)
        Method to attach function, which will be executed after encoder audio mode changed.

    send_command(command: ProntonetCommand)
        Send command to command protocol socket.

    """

    def __init__(self, ip, port=50031, status_port=50035):
        """
        Parameters
        ----------
        ip : str
            IP address of Prontonet device.
        port : int
            Port number of command protocol. (default is 50031)
        status_port : int
            Port number of status protocol. (default is 50035)
        """

        self.__command_socket: socket.socket or None = None
        self.__status_socket: socket.socket or None = None
        self.__status_th: threading.Thread or None = None
        self.ip: str = ip
        self.port: int = port
        self.status_port: int = status_port

        self.__on_connected: Callable or None = None
        self.__on_connected_args: tuple or None = None
        self.__on_status_socket_disconnected: Callable or None = None
        self.__on_status_socket_disconnected_args: tuple or None = None
        self.__on_status_socket_reconnected: Callable or None = None
        self.__on_status_socket_reconnected_args: tuple or None = None
        self.__on_line_status_changed: Callable or None = None
        self.__on_line_status_changed_args: tuple or None = None
        self.__on_alarm_status_changed: Callable or None = None
        self.__on_alarm_status_changed_args: tuple or None = None
        self.__on_decoder_audio_mode_changed: Callable or None = None
        self.__on_decoder_audio_mode_changed_args: tuple or None = None
        self.__on_encoder_audio_mode_changed: Callable or None = None
        self.__on_encoder_audio_mode_changed_args: tuple or None = None

    def connect(self) -> None:
        """
        Connects to status protocol socket and starts listening for messages.

        Returns
        -------
        None
        """

        self.__status_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        self.__status_socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
        self.__status_socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, 1)
        self.__status_socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, 1)
        self.__status_socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, 5)

        self.__status_socket.connect((self.ip, self.status_port))

        if self.__on_connected is not None:
            self.__on_connected(self.__on_connected_args)

        self.__status_th = threading.Thread(target=self.__status_loop, daemon=True)
        self.__status_th.start()

    def __status_loop(self) -> None:
        """
        Listens for status change. When socket disconnects, attempts to reconnect.

        Returns
        -------
        None
        """

        connected = True
        while True:
            if not connected:
                try:
                    self.__status_socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
                    self.__status_socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, 1)
                    self.__status_socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, 1)
                    self.__status_socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, 5)
                    self.__status_socket.connect((self.ip, self.status_port))
                    connected = True
                    time.sleep(2)
                    if self.__on_status_socket_reconnected is not None:
                        self.__on_status_socket_reconnected(self.__on_status_socket_reconnected_args)
                except socket.error:
                    time.sleep(5)
            else:
                try:
                    data = self.__status_socket.recv(1024)
                    self.__process_status_message(data)
                except socket.error:
                    if self.__on_status_socket_disconnected is not None:
                        self.__on_status_socket_disconnected(self.__on_status_socket_disconnected_args)
                    time.sleep(1)
                    self.__status_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    connected = False

    def __process_status_message(self, message: bytes) -> None:
        """
        Processes status messages. If function is attached to event, this method executes it.

        Parameters
        ----------
        message : bytes
            Message received from status socket.
        Returns
        -------
        None
        """

        if message[0] == Command.STATUS_LINE_STATUS_CHANGED:
            if self.__on_line_status_changed is not None:
                res = struct.unpack("<ii32sii", message[8:])
                num = ''.join(list(s for s in bytes(res[2]).decode("utf-8") if s.isprintable()))
                status = StatusLineStatusChanged(res[0], LineStatus(res[1]), num, res[3], IPCallType(res[4]))
                self.__on_line_status_changed(status, self.__on_line_status_changed_args)
        elif message[0] == Command.STATUS_ALARMS_STATUS_CHANGED:
            if self.__on_alarm_status_changed is not None:
                status = struct.unpack("<i", message[8:])
                self.__on_alarm_status_changed(StatusAlarmsStatusChanged(AlarmStatus(status[0])),
                                               self.__on_alarm_status_changed_args)
        elif message[0] == Command.STATUS_DECODER_AUDIO_MODE_CHANGED:
            if self.__on_decoder_audio_mode_changed is not None:
                message_len = len(message) - 12
                status = struct.unpack("<i" + str(message_len) + "s", message[8:])
                codec_name = ''.join(list(s for s in bytes(status[1]).decode("utf-8") if s.isprintable()))[:-1]
                self.__on_decoder_audio_mode_changed(StatusDecoderAudioModeChanged(Codec(status[0]), codec_name),
                                                     self.__on_decoder_audio_mode_changed_args)
        elif message[0] == Command.STATUS_ENCODER_AUDIO_MODE_CHANGED:
            if self.__on_encoder_audio_mode_changed is not None:
                message_len = len(message) - 12
                status = struct.unpack("<i" + str(message_len) + "s", message[8:])
                codec_name = ''.join(list(s for s in bytes(status[1]).decode("utf-8") if s.isprintable()))[:-1]
                self.__on_encoder_audio_mode_changed(StatusEncoderAudioModeChanged(Codec(status[0]), codec_name),
                                                     self.__on_encoder_audio_mode_changed_args)
        else:
            print("[!] Unknown status message received.")

    def disconnect(self) -> None:
        """
        Stops listening for status messages and closes status socket.

        Returns
        -------
        None
        """

        self.__status_th.join()
        self.__status_socket.close()

    def attach_on_connected(self, func, *args) -> None:
        """
        Method to attach function, which will be executed after status socket connected.

        Parameters
        ----------
        func
            Function to be executed when status socket is connected.
        args
            Function arguments.

        Returns
        -------
        None
        """

        self.__on_connected = func
        self.__on_connected_args = args

    def attach_on_status_socket_disconnected(self, func, *args) -> None:
        """
        Method to attach function, which will be executed after status socket disconnected.

        Parameters
        ----------
        func
            Function to be executed when status socket is disconnected.
        args
            Function arguments.

        Returns
        -------
        None
        """

        self.__on_status_socket_disconnected = func
        self.__on_status_socket_disconnected_args = args

    def attach_on_status_socket_reconnected(self, func, *args) -> None:
        """
        Method to attach function, which will be executed after status socket reconnected.

        Parameters
        ----------
        func
            Function to be executed when status socket is reconnected.
        args
            Function arguments.

        Returns
        -------
        None
        """

        self.__on_status_socket_reconnected = func
        self.__on_status_socket_reconnected_args = args

    def attach_on_line_status_changed(self, func, *args) -> None:
        """
        Method to attach function, which will be executed after line status changed.

        Parameters
        ----------
        func
            Function to be executed when line status changed.
        args
            Function arguments.

        Returns
        -------
        None
        """

        self.__on_line_status_changed = func
        self.__on_line_status_changed_args = args

    def attach_on_alarm_status_changed(self, func, *args) -> None:
        """
        Method to attach function, which will be executed after alarm status changed.

        Parameters
        ----------
        func
            Function to be executed when alarm status changed.
        args
            Function arguments.

        Returns
        -------
        None
        """

        self.__on_alarm_status_changed = func
        self.__on_alarm_status_changed_args = args

    def attach_on_decoder_audio_mode_changed(self, func, *args) -> None:
        """
        Method to attach function, which will be executed after decoder audio mode changed.

        Parameters
        ----------
        func
            Function to be executed when decoder audio mode changed.
        args
            Function arguments.

        Returns
        -------
        None
        """

        self.__on_decoder_audio_mode_changed = func
        self.__on_decoder_audio_mode_changed_args = args

    def attach_on_encoder_audio_mode_changed(self, func, *args) -> None:
        """
        Method to attach function, which will be executed after encoder audio mode changed.

        Parameters
        ----------
        func
            Function to be executed when encoder audio mode changed.
        args
            Function arguments.

        Returns
        -------
        None
        """

        self.__on_encoder_audio_mode_changed = func
        self.__on_encoder_audio_mode_changed_args = args

    def send_command(self, command: ProntonetCommand):
        """
        Sends command to command protocol socket.

        Parameters
        ----------
        command : ProntonetCommand
            Dataclass which contains command for Prontonet device (bytes),
            struct.unpack() pattern for response and command response type.

        Returns
        -------
        Structure passed to function as third argument of ProntonetCommand object.
        """

        self.__command_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__command_socket.connect((self.ip, self.port))
        self.__command_socket.send(ProntonetProtocol.connect())
        self.__command_socket.recv(1024)
        self.__command_socket.send(command.command)
        data = self.__command_socket.recv(2048)
        self.__command_socket.close()
        pattern = command.unpack_pattern
        if '#' in pattern:
            return command.response_type(data[8:-1])
        else:
            val = struct.unpack(pattern, data)
            return command.response_type(*val[2:])



import struct
from prontonet.structures import *
from prontonet.enums import *


class ProntonetProtocol:
    @staticmethod
    def connect():
        """
        Creates initial connect message.

        Returns
        -------
        bytes
            Connect message.
        """

        command_code = Command.CONNECT
        cs = CSConnect.securityString
        b = struct.pack("<ii", command_code, 20)
        return b + cs.zfill(20)

    @staticmethod
    def get_device_net() -> ProntonetCommand:
        """
        Get device network.

        Returns
        -------
        ProntonetCommand
            Dataclass with get device network message, struct.unpack() pattern for response and response type.
        """
        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_DEVICE_NET, 0),
            "iii",
            DeviceNet
        )

    @staticmethod
    def set_device_net(arg: CommandDeviceNet) -> ProntonetCommand:
        """
        Set device network.

        Parameters
        ----------
        arg : CommandDeviceNet
            Settings to be sent to device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set device network message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_SET_DEVICE_NET, 4, arg.device_net),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def call(arg: CommandCall) -> ProntonetCommand:
        """
        Call.

        Parameters
        ----------
        arg : CommandCall
            Call settings.

        Returns
        -------
        ProntonetCommand
            Dataclass with call message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_CALL, 40, arg.line)
            + arg.number[0:32].ljust(32, '\x00').encode("utf-8")
            + struct.pack("<i", arg.ip_call_type),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def answer(arg: CommandAnswer) -> ProntonetCommand:
        """
        Answer a call.

        Parameters
        ----------
        arg : CommandAnswer
            Answer command settings.

        Returns
        -------
        ProntonetCommand
            Dataclass with answer message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ANSWER, 4, arg.line),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def hang_up(arg: CommandHangUp) -> ProntonetCommand:
        """
        Hang up a call.

        Parameters
        ----------
        arg : CommandHangUp
            Hang up command settings.

        Returns
        -------
        ProntonetCommand
            Dataclass with hang up message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_HANG_UP, 4, arg.line),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_line_status(arg: CommandGetLineStatus) -> ProntonetCommand:
        """
        Get the current line status.

        Parameters
        ----------
        arg : CommandGetLineStatus
            Command settings (line number).

        Returns
        -------
        ProntonetCommand
            Dataclass with get line status message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_GET_LINE_STATUS, 4, arg.line),
            "iii",
            CommandGetLineStatusResponse
        )

    @staticmethod
    def get_line_status_details(arg: CommandGetLineStatusDetails) -> ProntonetCommand:
        """
        Get the current line status details.

        Parameters
        ----------
        arg : CommandGetLineStatus
            Command settings (line number).

        Returns
        -------
        ProntonetCommand
            Dataclass with get line status details message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_GET_LINE_STATUS_DETAILS, 4, arg.line),
            "ii32sii",
            CommandGetLineStatusDetailsResponse
        )

    @staticmethod
    def get_vu_meters() -> ProntonetCommand:
        """
        Get current input and output VU meters.

        Returns
        -------
        ProntonetCommand
            Dataclass with get VU meters message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_VU_METERS, 0),
            "iiiiii",
            CommandGetVUMetersResponse
        )

    @staticmethod
    def get_monitors() -> ProntonetCommand:
        """
        Get current system monitor values (hardware diagnostic data).

        Returns
        -------
        ProntonetCommand
            Dataclass with get monitors message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_MONITORS, 0),
            "iiiiii",
            CommandGetMonitorsResponse
        )

    @staticmethod
    def get_alarm_status() -> ProntonetCommand:
        """
        Get current alarm status.

        Returns
        -------
        ProntonetCommand
            Dataclass with get alarm status message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_ALARMS_GET_STATUS, 0),
            "iii",
            AlarmStatus
        )

    @staticmethod
    def get_decoder_audio_mode(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get decoder's audio mode.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command settings (codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE, 4, arg.codec),
            "ii#",
            bytes
        )

    @staticmethod
    def get_encoder_audio_mode(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get encoder's audio mode.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command settings (codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE, 4, arg.codec),
            "ii#",
            bytes
        )

    @staticmethod
    def set_encoder_audio_mode_pcm(arg: CommandEncoderSetAudioModePCM) -> ProntonetCommand:
        """
        Set encoder's audio mode to PCM.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModePCM
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode PCM message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii4i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_PCM, 16,
                        arg.codec, arg.bits_sample, arg.audio_mode, arg.encoder_mix),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_mpeg(arg: CommandEncoderSetAudioModeMPEG) -> ProntonetCommand:
        """
        Set encoder's audio mode to MPEG.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeMPEG
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode MPEG message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii9i", Command.COMMAND_ENCODER_GET_AUDIO_MODE_MPEG, 36,
                        arg.codec, arg.bit_rate, arg.audio_mode, arg.mpeg_layer, arg.frequency, arg.crc,
                        arg.aux_data, arg.encoder_mix, arg.bonding_type),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_aac(arg: CommandEncoderSetAudioModeAAC) -> ProntonetCommand:
        """
        Set encoder's audio mode to AAC.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeAAC
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode AAC message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii9i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_AAC, 36,
                        arg.codec, arg.bit_rate, arg.audio_mode, arg.aac_mode, arg.frequency, arg.crc,
                        arg.aux_data, arg.encoder_mix, arg.bonding_type),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_g711(arg: CommandEncoderSetAudioModeG711) -> ProntonetCommand:
        """
        Set encoder's audio mode to G711.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeG711
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode G711 message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii2i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_G711, 8,
                        arg.codec, arg.encoder_mix),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_g722(arg: CommandEncoderSetAudioModeG722) -> ProntonetCommand:
        """
        Set encoder's audio mode to G722.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeG722
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode G722 message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii2i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_G711, 8,
                        arg.codec, arg.encoder_mix),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_auto(arg: CommandEncoderSetAudioModeAuto) -> ProntonetCommand:
        """
        Set encoder's audio mode to AUTO.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeAuto
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode AUTO message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii2i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_AUTO, 8,
                        arg.codec, arg.aux_data),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_aptx(arg: CommandEncoderSetAudioModeAPTX) -> ProntonetCommand:
        """
        Set encoder's audio mode to APTX.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeAPTX
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode APTX message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii7i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_APTX, 28,
                        arg.codec, arg.audio_mode, arg.aptx_type, arg.frequency,
                        arg.crc, arg.aux_data, arg.encoder_mix),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_g711_v2(arg: CommandEncoderSetAudioModeG711v2) -> ProntonetCommand:
        """
        Set encoder's audio mode to G711.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeG711v2
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode G711v2 message, struct.unpack() pattern for response
            and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii3i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_G711_V2, 12,
                        arg.codec, arg.encoder_mix, arg.g711_law),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_g722_v2(arg: CommandEncoderSetAudioModeG722v2) -> ProntonetCommand:
        """
        Set encoder's audio mode to G722.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeG722v2
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode G722v2 message, struct.unpack() pattern for response
            and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii3i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_G722_V2, 12,
                        arg.codec, arg.encoder_mix, arg.g722_dither),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_mpeg_v2(arg: CommandEncoderSetAudioModeMPEGv2) -> ProntonetCommand:
        """
        Set encoder's audio mode to MPEG.

        Parameters
        ----------
        arg : CommandEncoderSetAudioModeMPEGv2
            Settings to be applied on device.

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode MPEGv2 message, struct.unpack() pattern for response
            and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii10i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_MPEG_V2, 40,
                        arg.codec, arg.bit_rate, arg.audio_mode, arg.mpeg_layer, arg.frequency,
                        arg.crc, arg.aux_data, arg.encoder_mix, arg.bonding_type, arg.mpeg_l3_polarity),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_ip_streaming_protocol() -> ProntonetCommand:
        """
        Get actual IP streaming protocol.

        Returns
        -------
        ProntonetCommand
            Dataclass with get IP streaming protocol message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_IP_GET_STREAMING_PROTOCOL, 0),
            "iii",
            StreamingProtocol
        )

    @staticmethod
    def set_ip_streaming_protocol(arg: CommandIPSetStreamingProtocol) -> ProntonetCommand:
        """
        Set IP streaming protocol.

        Parameters
        ----------
        arg : CommandIPSetStreamingProtocol
            Settings to be applied.

        Returns
        -------
        ProntonetCommand
            Dataclass with set IP streaming protocol message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_IP_SET_STREAMING_PROTOCOL, 4, arg.streaming_protocol),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def x21_enable_tx(arg: CommandX21EnableTX) -> ProntonetCommand:
        """
        Enable or disable the transmission of the X21 interface (clock + data).

        Parameters
        ----------
        arg : CommandX21EnableTX
            Settings to be applied.

        Returns
        -------
        ProntonetCommand
            Dataclass with x21 enable/disable message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_IP_SET_STREAMING_PROTOCOL, 4, arg.enable),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def load_preset(arg: CommandLoadPreset) -> ProntonetCommand:
        """
        Load and apply the specified preset.

        Parameters
        ----------
        arg : CommandLoadPreset
            Settings to be applied.

        Returns
        -------
        ProntonetCommand
            Dataclass with load preset message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_LOAD_PRESET, 255)
            + arg.name[0:255].ljust(255, '\x00').encode("utf-8"),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def call_from_book(arg: CommandCallFromBook) -> ProntonetCommand:
        """
        Make a call using phonebook.

        Parameters
        ----------
        arg : CommandCallFromBook
            Dataclass with line number and book entry.

        Returns
        -------
        ProntonetCommand
            Dataclass with call from book message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii2i", Command.COMMAND_CALL_FROM_BOOK, 8, arg.line, arg.book_entry),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def alarms_ack() -> ProntonetCommand:
        """
        Acknowledge all the alarms from the alarms history.

        Returns
        -------
        ProntonetCommand
            Dataclass with alarms acknowledge message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_ALARMS_ACK, 0),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def alarms_delete() -> ProntonetCommand:
        """
        Delete all the alarms from the alarms history.

        Returns
        -------
        ProntonetCommand
            Dataclass with alarms delete message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_ALARMS_DELETE, 0),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def call_from_book_all_lines(arg: CommandCallFromBookAllLines) -> ProntonetCommand:
        """
        Make a call using phonebook to all the lines specified in the phonebook.

        Parameters
        ----------
        arg : CommandCallFromBookAllLines
            Dataclass which specifies phonebook entry ID.

        Returns
        -------
        ProntonetCommand
            Dataclass with call from book using all lines message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_CALL_FROM_BOOK_ALL_LINES, 4, arg.book_entry),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_device_net_v2() -> ProntonetCommand:
        """
        Get the network in use and its parameters.

        Returns
        -------
        ProntonetCommand
            Dataclass with get device network V2 message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_DEVICE_NET_V2, 0),
            "ii5i",
            CommandDeviceNetV2
        )

    @staticmethod
    def set_device_net_v2(arg: CommandDeviceNetV2) -> ProntonetCommand:
        """
        Set the network and its parameters.
        Several restrictions apply to the parameters (not all the combinations are allowed).

        Parameters
        ----------
        arg : CommandDeviceNetV2
            Dataclass which contains DeviceNet, DeviceSubNet, ProntonetCodecMode, MultiUnicast
            and StreamingProtocol settings.

        Returns
        -------
        ProntonetCommand
            Dataclass with set device network V2 message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii5i", Command.COMMAND_SET_DEVICE_NET_V2, 20, arg.device_net, arg.device_sub_net,
                        arg.pronto_net_codec_mode, arg.multi_unicast, arg.streaming_protocol),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def call_v2(arg: CommandCallV2) -> ProntonetCommand:
        """
        Make a call. Current connection (if any) will be disconnected before trying the new call.

        Parameters
        ----------
        arg : CommandCallV2
            Dataclass which contains line number, callee number, IPCallType and Codec settings.

        Returns
        -------
        ProntonetCommand
            Dataclass with call V2 message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_CALL_V2, 44, arg.line)
            + arg.number[0:32].ljust(32, '\x00').encode("utf-8")
            + struct.pack("<ii", arg.ip_call_type, arg.target_codec),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_loaded_preset_index() -> ProntonetCommand:
        """
        Get the index of the current loaded preset.

        Returns
        -------
        ProntonetCommand
            Dataclass with get loaded preset index message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_LOADED_PRESET_INDEX, 0),
            "iii",
            CommandGetLoadedPresetIndex
        )

    @staticmethod
    def run_gpi_action(arg: CommandRunGPIAction) -> ProntonetCommand:
        """
        Execute a GPI action.

        Parameters
        ----------
        arg : CommandRunGPIAction
            Dataclass which contains GPI number and active/not active information.

        Returns
        -------
        ProntonetCommand
            Dataclass with run GPI action message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii2i", Command.COMMAND_RUN_GPI_ACTION, 8, arg.gpi, arg.active),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def enable_net_backup(arg: CommandEnableNetBackup) -> ProntonetCommand:
        """
        Enable/disable the network backup feature.

        Parameters
        ----------
        arg : CommandEnableNetBackup
            Settings to be applied.

        Returns
        -------
        ProntonetCommand
            Dataclass with enable/disable network backup message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENABLE_NET_BACKUP, 4, arg.enable),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def reset_device() -> ProntonetCommand:
        """
        Reboot the system.

        Returns
        -------
        ProntonetCommand
            Dataclass with reset device message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_RESET_DEVICE, 0),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_loaded_preset_name() -> ProntonetCommand:
        """
        Get the name of the last loaded preset configuration.

        Returns
        -------
        ProntonetCommand
            Dataclass with get loaded preset name message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_LOADED_PRESET_NAME, 0),
            "#",
            bytes
        )

    @staticmethod
    def get_streaming_stats(arg: CommandGetStreamingStats) -> ProntonetCommand:
        """
        Get streaming statistics for a specific codec.

        Parameters
        ----------
        arg : CommandGetStreamingStats
            Command settings (codec index).

        Returns
        -------
        ProntonetCommand
            Dataclass with get streaming statistics message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_GET_STREAMING_STATS, 20, arg.target_codec),
            "ii4i",
            CommandGetStreamingStatsResponse
        )

    @staticmethod
    def get_sip_server_register_status() -> ProntonetCommand:
        """
        Get the status of the SIP server.

        Returns
        -------
        ProntonetCommand
            Dataclass with get SIP server registration status message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_SIP_SERVER_REGISTER_STATUS, 0),
            "iii",
            CommandGetSIPServerRegisterStatus
        )

    @staticmethod
    def get_stun_server_register_status() -> ProntonetCommand:
        """
        Get the status of the STUN server.

        Returns
        -------
        ProntonetCommand
            Dataclass with get STUN server registration status message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_GET_STUN_SERVER_REGISTER_STATUS, 0),
            "iii",
            CommandGetSTUNServerRegisterStatus
        )

    @staticmethod
    def get_call_duration(arg: CommandGetCallDuration) -> ProntonetCommand:
        """
        Get the current duration of one active call in seconds.

        Parameters
        ----------
        arg : CommandGetCallDuration
            Command parameter (line number).

        Returns
        -------
        ProntonetCommand
            Dataclass with get call duration message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_GET_CALL_DURATION, 4, arg.line),
            "iii",
            CommandGetCallDurationResponse
        )

    @staticmethod
    def alarms_get_log() -> ProntonetCommand:
        """
        Get machine's alarms log as string which can be saved in an XML file.

        Returns
        -------
        ProntonetCommand
            Dataclass with get alarms log message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_ALARMS_GET_LOG, 0),
            "ii#",
            bytes
        )

    @staticmethod
    def get_audio_configuration() -> ProntonetCommand:
        """
        Get the machine's audio configuration (only if it is stationary unit, not a portable one).

        Returns
        -------
        ProntonetCommand
            Dataclass with get audio configuration message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_AUDIO_GET_CONFIGURATION, 0),
            "ii6i",
            CommandAudioGetConfigurationResponse
        )

    @staticmethod
    def set_audio_configuration(arg: CommandAudioSetConfiguration) -> ProntonetCommand:
        """
        Set the machine's audio configuration (only if it is stationary unit, not a portable one).

        Parameters
        ----------
        arg : CommandAudioSetConfiguration
            Settings to be applied (AudioConfig - the audio mode, dB left input, dB right input, dB left output,
            dB right output).

        Returns
        -------
        ProntonetCommand
            Dataclass with set audio configuration message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii5i", Command.COMMAND_AUDIO_SET_CONFIGURATION, 20,
                        arg.audio_config, arg.db_in_left, arg.db_in_right, arg.db_out_left, arg.db_out_right),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_device_name() -> ProntonetCommand:
        """
        Get device's configured name.

        Returns
        -------
        ProntonetCommand
            Dataclass with get device's name message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_SYS_GET_DEVICE_NAME, 0),
            "ii#",
            bytes
        )

    @staticmethod
    def set_device_name(arg: CommandSysDeviceName) -> ProntonetCommand:
        """
        Set the device name.

        Parameters
        ----------
        arg : CommandSysDeviceName
            Dataclass which contains new device name.

        Returns
        -------
        ProntonetCommand
            Dataclass with set device's name message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_SYS_SET_DEVICE_NAME, 256)
            + arg.device_name[0:256].ljust(256, '\x00').encode("utf-8"),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def blink_device() -> ProntonetCommand:
        """
        Blink the front LEDs of the unit for some seconds.

        Returns
        -------
        ProntonetCommand
            Dataclass with blink device message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_SYS_BLINK_DEVICE, 0),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_version_info() -> ProntonetCommand:
        """
        Get the device's serial number and software version.

        Returns
        -------
        ProntonetCommand
            Dataclass with get version info message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_SYS_GET_VERSION_INFO, 0),
            "ii16s16s",
            CommandSysGetVersionInfo
        )

    @staticmethod
    def get_sip_configuration() -> ProntonetCommand:
        """
        Get the configuration parameters of the SIP protocol from the unit.

        Returns
        -------
        ProntonetCommand
            Dataclass with get SIP configuration message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_SIP_GET_CONFIGURATION, 0),
            "iiiiiiiii256s256s256s256sii256s256si",
            CommandSIPConfiguration
        )

    @staticmethod
    def set_sip_configuration(arg: CommandSIPConfiguration) -> ProntonetCommand:
        """
        Set the configuration parameters of the SIP protocol of the unit.

        Parameters
        ----------
        arg : CommandSIPConfiguration
            SIP settings to be applied.

        Returns
        -------
        ProntonetCommand
            Dataclass with set SIP configuration message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_SIP_SET_CONFIGURATION, 1576)
            + struct.pack("<7i", arg.sip_port, arg.audio_port, arg.auto_answer, arg.enable_fec, arg.fec_port,
                          arg.audio_packets_per_fec_packet, arg.register_in_server)
            + arg.sip_user_name[0:256].ljust(256, '\x00').encode("utf-8")
            + arg.sip_server_address[0:256].ljust(256, '\x00').encode("utf-8")
            + arg.sip_server_user[0:256].ljust(256, '\x00').encode("utf-8")
            + arg.sip_server_password[0:256].ljust(256, '\x00').encode("utf-8")
            + struct.pack("<ii", arg.sip_server_timeout, arg.sip_address_type)
            + arg.public_address[0:256].ljust(256, '\x00').encode("utf-8")
            + arg.stun_address[0:256].ljust(256, '\x00').encode("utf-8")
            + struct.pack("<i", arg.stun_request_period),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_configuration_v4() -> ProntonetCommand:
        """
        Get the configuration parameters of the Prodys' Proprietary Protocol V4 from the unit.

        Returns
        -------
        ProntonetCommand
            Dataclass with get PP V4 configuration message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii", Command.COMMAND_PRODYS_V4_GET_CONFIGURATION, 0),
            "ii6i",
            CommandProdysV4ConfigurationResponse
        )

    @staticmethod
    def set_configuration_v4(arg: CommandProdysV4Configuration) -> ProntonetCommand:
        """
        Set the configuration parameters of the Prodys' Proprietary Protocol V4 of the unit.

        Parameters
        ----------
        arg : CommandProdysV4Configuration
            Settings to be applied.

        Returns
        -------
        ProntonetCommand
            Dataclass with set PP V4 configuration message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii6i", Command.COMMAND_PRODYS_V4_SET_CONFIGURATION, 24, arg.line_1_port,
                        arg.enable_control_port, arg.auto_answer, arg.enable_fec, arg.audio_packets_per_fec_packet,
                        arg.fec_delay),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_streaming_tx_codec_configuration(arg: CommandStreamingTXCodecGetConfiguration) -> ProntonetCommand:
        """
        Get the configuration parameters of the streaming TX for a given codec and audio algorithm.

        Parameters
        ----------
        arg : CommandStreamingTXCodecGetConfiguration
            Command parameters (Codec and AudioAlgorithm).

        Returns
        -------
        ProntonetCommand
            Dataclass with get streaming TX codec configuration message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii2i", Command.COMMAND_STREAMING_TX_GET_CODEC_CONFIGURATION,
                        arg.codec, arg.audio_algorithm),
            "iiii",
            CommandStreamingTXCodecGetConfigurationResponse
        )

    @staticmethod
    def set_streaming_tx_codec_configuration(arg: CommandStreamingTXCodecSetConfiguration) -> ProntonetCommand:
        """
        Set the configuration parameters of the streaming TX for a given codec.

        Parameters
        ----------
        arg : CommandStreamingTXCodecSetConfiguration
            Command parameters (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with set streaming TX codec configuration message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii3i", Command.COMMAND_STREAMING_TX_SET_CODEC_CONFIGURATION,
                        arg.codec, arg.audio_algorithm, arg.time_between_packets),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_streaming_rx_codec_configuration(arg: CommandStreamingRXCodecGetConfiguration) -> ProntonetCommand:
        """
        Get the configuration parameters of the streaming RX for a given codec.

        Parameters
        ----------
        arg : CommandStreamingRXCodecGetConfiguration
            Command parameters (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get streaming RX codec configuration message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_STREAMING_RX_GET_CODEC_CONFIGURATION, 4, arg.codec),
            "ii7i",
            CommandStreamingRXCodecGetConfigurationResponse
        )

    @staticmethod
    def set_streaming_rx_codec_configuration(arg: CommandStreamingRXCodecSetConfiguration) -> ProntonetCommand:
        """
        Set the configuration parameters of the streaming RX for a given codec.

        Parameters
        ----------
        arg : CommandStreamingRXCodecSetConfiguration
            Settings to be applied.

        Returns
        -------
        ProntonetCommand
            Dataclass with set streaming RX codec configuration message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii7i", Command.COMMAND_STREAMING_RX_SET_CODEC_CONFIGURATION, 28,
                        arg.codec, arg.auto_adjustment, arg.auto_min_delay, arg.auto_min_delay_duration,
                        arg.auto_max_delay, arg.auto_max_delay_duration, arg.manual_max_delay),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_streaming_stats_v2(arg: CommandGetStreamingStatsV2) -> ProntonetCommand:
        """
        Get the extended streaming statistics for a given codec.

        Parameters
        ----------
        arg : CommandGetStreamingStatsV2
            Command parameters (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get streaming statistics V2 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_GET_STREAMING_STATS_V2, 4, arg.codec),
            "ii10i",
            CommandGetStreamingStatsV2Response
        )

    @staticmethod
    def get_ethernet_speed(arg: CommandEthernetGetSpeed) -> ProntonetCommand:
        """
        Get the current negotiated speed on a given Ethernet interface.

        Parameters
        ----------
        arg : CommandEthernetGetSpeed
            Command parameters (EthernetInterface).

        Returns
        -------
        ProntonetCommand
            Dataclass with get ethernet speed message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ETHERNET_GET_SPEED, 4, arg.ethernet_interface),
            "ii3i",
            CommandEthernetGetSpeedResponse
        )

    @staticmethod
    def streaming_stats_reset(arg: CommandStreamingStatsReset) -> ProntonetCommand:
        """
        Clear the full streaming statistics for a given codec, or both.

        Parameters
        ----------
        arg : CommandStreamingStatsReset
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with reset streaming statistics message, struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_STREAMING_STATS_RESET, 4, arg.codec),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_pcm_v2(arg: CommandAudioModeConfigPCMV2) -> ProntonetCommand:
        """
        Set the audio configuration for a given codec with PCM algorithm.

        Parameters
        ----------
        arg : CommandAudioModeConfigPCMV2
            Settings to be applied (Codec, BitsSample, AudioMode, EncoderMix, AuxData).

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode PCM V2 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii5i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_PCM_V2, 20,
                        arg.codec, arg.bits_sample, arg.audio_mode, arg.encoder_mix, arg.aux_data),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_g711_v3(arg: CommandAudioModeConfigG711V3) -> ProntonetCommand:
        """
        Set the audio configuration for a given codec with G711 algorithm.

        Parameters
        ----------
        arg : CommandAudioModeConfigG711V3
            Settings to be applied (Codec, G711Law, EncoderMix, AuxData).

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode G711 V3 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii4i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_G711_V3, 16,
                        arg.codec, arg.g711_law, arg.encoder_mix, arg.aux_data),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_g722_v3(arg: CommandAudioModeConfigG722V3) -> ProntonetCommand:
        """
        Set the audio configuration for a given codec with G722 algorithm.

        Parameters
        ----------
        arg : CommandAudioModeConfigG722V3
            Settings to be applied (Codec, EncoderMix, AuxData, G722Dither).

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode G722 V3 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii4i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_G722_V3, 16,
                        arg.codec, arg.encoder_mix, arg.aux_data, arg.g722_dither),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def set_encoder_audio_mode_aptx_v2(arg: CommandAudioModeConfigAPTXV2) -> ProntonetCommand:
        """
        Set the audio configuration for a given codec with APTX algorithm.

        Parameters
        ----------
        arg : CommandAudioModeConfigAPTXV2
            Settings to be applied (Codec, APTXType, BitRate, AudioMode, EncoderMix, AuxData).

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode APTX V2 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii6i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_APTX_V2, 24,
                        arg.codec, arg.aptx_type, arg.bit_rate, arg.audio_mode, arg.encoder_mix, arg.aux_data),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_encoder_audio_mode_algorithm(arg: CommandGetAudioModeAlgorithm) -> ProntonetCommand:
        """
        Get the current audio algorithm family configured on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandGetAudioModeAlgorithm
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode algorithm message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_ALGORITHM, 4, arg.codec),
            "ii2i",
            CommandEncoderGetAudioModeAlgorithmResponse
        )

    @staticmethod
    def get_decoder_audio_mode_algorithm(arg: CommandGetAudioModeAlgorithm) -> ProntonetCommand:
        """
        Get the current audio algorithm family configured on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandGetAudioModeAlgorithm
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode algorithm message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_ALGORITHM, 4, arg.codec),
            "ii3i",
            CommandDecoderGetAudioModeAlgorithmResponse
        )

    @staticmethod
    def get_encoder_audio_mode_auto(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current AUTO configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode auto message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_AUTO, 4, arg.codec),
            "ii2i",
            CommandEncoderGetAudioModeAutoResponse
        )

    @staticmethod
    def get_encoder_audio_mode_pcm(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current PCM configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode PCM message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_PCM, 4, arg.codec),
            "ii5i",
            CommandEncoderGetAudioModePCMResponse
        )

    @staticmethod
    def get_encoder_audio_mode_g711(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current G711 configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode G711 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_G711, 4, arg.codec),
            "ii4i",
            CommandEncoderGetAudioModeG711Response
        )

    @staticmethod
    def get_encoder_audio_mode_g722(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current G722 configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode G722 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_G722, 4, arg.codec),
            "ii4i",
            CommandEncoderGetAudioModeG722Response
        )

    @staticmethod
    def get_encoder_audio_mode_mpeg(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current MPEG configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode MPEG message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_MPEG, 4, arg.codec),
            "ii10i",
            CommandEncoderGetAudioModeMPEGResponse
        )

    @staticmethod
    def get_encoder_audio_mode_acc(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current AAC configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode AAC message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_AAC, 4, arg.codec),
            "ii9i",
            CommandEncoderGetAudioModeAACResponse
        )

    @staticmethod
    def get_encoder_audio_mode_aptx(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current APTX configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode APTX message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_APTX, 4, arg.codec),
            "ii6i",
            CommandEncoderGetAudioModeAPTXResponse
        )

    @staticmethod
    def get_decoder_audio_mode_pcm(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current PCM configuration details on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode PCM message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_PCM, 4, arg.codec),
            "ii5i",
            CommandDecoderGetAudioModePCMResponse
        )

    @staticmethod
    def get_decoder_audio_mode_g711(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current G711 configuration details on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode G711 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_G711, 4, arg.codec),
            "ii4i",
            CommandDecoderGetAudioModeG711Response
        )

    @staticmethod
    def get_decoder_audio_mode_g722(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current G722 configuration details on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode G722 message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_G722, 4, arg.codec),
            "ii4i",
            CommandDecoderGetAudioModeG722Response
        )

    @staticmethod
    def get_decoder_audio_mode_mpeg(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current MPEG configuration details on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode MPEG message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_MPEG, 4, arg.codec),
            "ii10i",
            CommandDecoderGetAudioModeMPEGResponse
        )

    @staticmethod
    def get_decoder_audio_mode_aac(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current AAC configuration details on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode AAC message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_AAC, 4, arg.codec),
            "ii9i",
            CommandDecoderGetAudioModeAACResponse
        )

    @staticmethod
    def get_decoder_audio_mode_aptx(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current APTX configuration details on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode APTX message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_APTX, 4, arg.codec),
            "ii6i",
            CommandDecoderGetAudioModeAPTXResponse
        )

    @staticmethod
    def set_encoder_audio_mode_opus(arg: CommandAudioModeOPUS) -> ProntonetCommand:
        """
        Set the audio configuration for a given codec with an OPUS algorithm.

        Parameters
        ----------
        arg : CommandAudioModeOPUS
            Command parameter (Codec, BitRate, EncoderMix, AuxData).

        Returns
        -------
        ProntonetCommand
            Dataclass with set encoder audio mode OPUS message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<ii4i", Command.COMMAND_ENCODER_SET_AUDIO_MODE_OPUS, 16,
                        arg.codec, arg.bit_rate, arg.encoder_mix, arg.aux_data),
            "iiii",
            AcknowledgeResponse
        )

    @staticmethod
    def get_encoder_audio_mode_opus(arg: CommandEncoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current OPUS configuration details on the encoder for a given codec.

        Parameters
        ----------
        arg : CommandEncoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get encoder audio mode OPUS message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_ENCODER_GET_AUDIO_MODE_OPUS, 4, arg.codec),
            "ii4i",
            CommandAudioModeOPUS
        )

    @staticmethod
    def get_decoder_audio_mode_opus(arg: CommandDecoderGetAudioMode) -> ProntonetCommand:
        """
        Get the current OPUS configuration details on the decoder for a given codec.

        Parameters
        ----------
        arg : CommandDecoderGetAudioMode
            Command parameter (Codec).

        Returns
        -------
        ProntonetCommand
            Dataclass with get decoder audio mode OPUS message,
            struct.unpack() pattern for response and response type.
        """

        return ProntonetCommand(
            struct.pack("<iii", Command.COMMAND_DECODER_GET_AUDIO_MODE_OPUS, 4, arg.codec),
            "ii4i",
            CommandAudioModeOPUS
        )



from dataclasses import dataclass
from typing import Type
from prontonet.constants import IP_SECURITY_STRING
from prontonet.enums import Command, DeviceNet, LineStatus, IPCallType, AlarmStatus, Codec, BitsSample, AudioMode, \
    EncoderMix, MPEGLayer, Frequency, BondingType, AACMode, BitRate, APTXType, G711Law, MPEGL3Polarity, \
    StreamingProtocol, DeviceSubNet, ProntoNetCodecMode, AudioConfig, SIPAddressType, AudioAlgorithm, \
    EthernetInterface, EthernetNegotiatedSpeed, AudioModeAlgorithm, Acknowledge


@dataclass
class ProntonetCommand:
    command: bytes
    unpack_pattern: str
    response_type: Type


@dataclass
class AcknowledgeResponse:
    command: Command
    acknowledge: Acknowledge


@dataclass
class CSConnect:
    securityString = bytes(IP_SECURITY_STRING, "utf-8")


@dataclass
class CSCommandResponse:
    command: int
    response: int


@dataclass
class StatusDeviceNetChanged:
    device_net: DeviceNet


@dataclass
class StatusLineStatusChanged:
    line: int
    line_status: LineStatus
    number: str
    disconnection_code: int
    ip_call_type: IPCallType


@dataclass
class StatusAlarmsStatusChanged:
    alarm_status: AlarmStatus


@dataclass
class StatusDecoderAudioModeChanged:
    codec: Codec
    name: str


@dataclass
class StatusEncoderAudioModeChanged:
    codec: Codec
    name: str


@dataclass
class CommandDeviceNet:
    device_net: DeviceNet


@dataclass
class CommandCall:
    line: int
    number: str
    ip_call_type: IPCallType


@dataclass
class CommandHangUp:
    line: int


@dataclass
class CommandGetLineStatus:
    line: int


@dataclass
class CommandGetLineStatusResponse:
    line_status: LineStatus


@dataclass
class CommandGetVUMetersResponse:
    input_left_vu_meter: int
    input_right_vu_meter: int
    output_left_vu_meter: int
    output_right_vu_meter: int


@dataclass
class CommandGetMonitorsResponse:
    v12_main: int
    v12_backup: int
    temp: int
    fan_rpm: int


@dataclass
class CommandAlarmsGetStatusResponse:
    alarm_status: AlarmStatus


@dataclass
class CommandDecoderGetAudioMode:
    codec: Codec


@dataclass
class CommandEncoderGetAudioMode:
    codec: Codec


@dataclass
class CommandEncoderSetAudioModePCM:
    codec: Codec
    bits_sample: BitsSample
    audio_mode: AudioMode
    encoder_mix: EncoderMix


@dataclass
class CommandEncoderSetAudioModeMPEG:
    codec: Codec
    bit_rate: BitRate
    audio_mode: AudioMode
    mpeg_layer: MPEGLayer
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix
    bonding_type: BondingType


@dataclass
class CommandEncoderSetAudioModeAAC:
    codec: Codec
    bit_rate: BitRate
    audio_mode: AudioMode
    aac_mode: AACMode
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix
    bonding_type: BondingType


@dataclass
class CommandEncoderSetAudioModeG711:
    codec: Codec
    encoder_mix: EncoderMix


@dataclass
class CommandEncoderSetAudioModeG722:
    codec: Codec
    encoder_mix: EncoderMix


@dataclass
class CommandEncoderSetAudioModeAuto:
    codec: Codec
    aux_data: bool


@dataclass
class CommandEncoderSetAudioModeAPTX:
    codec: Codec
    audio_mode: AudioMode
    aptx_type: APTXType
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix


@dataclass
class CommandGetLineStatusDetails:
    line: int


@dataclass
class CommandGetLineStatusDetailsResponse:
    number: str
    disconnection_code: int
    ip_call_type: IPCallType


@dataclass
class CommandAnswer:
    line: int


@dataclass
class CommandEncoderSetAudioModeG711v2:
    codec: Codec
    encoder_mix: EncoderMix
    g711_law: G711Law


@dataclass
class CommandEncoderSetAudioModeG722v2:
    codec: Codec
    encoder_mix: EncoderMix
    g722_dither: bool


@dataclass
class CommandEncoderSetAudioModeMPEGv2:
    codec: Codec
    bit_rate: BitRate
    audio_mode: AudioMode
    mpeg_layer: MPEGLayer
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix
    bonding_type: BondingType
    mpeg_l3_polarity: MPEGL3Polarity


@dataclass
class CommandIPSetStreamingProtocol:
    streaming_protocol: StreamingProtocol


@dataclass
class CommandX21EnableTX:
    enable: bool


@dataclass
class CommandLoadPreset:
    name: str


@dataclass
class CommandCallFromBook:
    line: int
    book_entry: int


@dataclass
class CommandCallFromBookAllLines:
    book_entry: int


@dataclass
class CommandDeviceNetV2:
    device_net: DeviceNet
    device_sub_net: DeviceSubNet
    pronto_net_codec_mode: ProntoNetCodecMode
    multi_unicast: bool
    streaming_protocol: StreamingProtocol


@dataclass
class CommandCallV2:
    line: int
    number: str
    ip_call_type: IPCallType
    target_codec: Codec


@dataclass
class CommandEnableNetBackup:
    enable: bool


@dataclass
class CommandGetLoadedPresetIndex:
    index: int


@dataclass
class CommandRunGPIAction:
    gpi: int
    active: bool


@dataclass
class CommandGetLoadedPresetName:
    last_loaded_preset_name: str


@dataclass
class CommandGetStreamingStats:
    target_codec: Codec


@dataclass
class CommandGetStreamingStatsResponse:
    lost_packets: int
    disordered_packets: int
    recovered_packets: int
    current_jitter: int


@dataclass
class CommandGetSIPServerRegisterStatus:
    registered: bool


@dataclass
class CommandGetSTUNServerRegisterStatus:
    registered: bool


@dataclass
class CommandGetCallDuration:
    line: int


@dataclass
class CommandGetCallDurationResponse:
    duration_seconds = int


@dataclass
class CommandAudioGetConfigurationResponse:
    command_supported: bool
    audio_config: AudioConfig
    db_in_left: int
    db_in_right: int
    db_out_left: int
    db_out_right: int


@dataclass
class CommandAudioSetConfiguration:
    audio_config: AudioConfig
    db_in_left: int
    db_in_right: int
    db_out_left: int
    db_out_right: int


@dataclass
class CommandSysDeviceName:
    device_name: str


@dataclass
class CommandSysGetVersionInfo:
    serial_number: bytes
    version: bytes


@dataclass
class CommandSIPConfiguration:
    sip_port: int
    audio_port: int
    auto_answer: bool
    enable_fec: bool
    fec_port: int
    audio_packets_per_fec_packet: int
    register_in_server: bool
    sip_user_name: str
    sip_server_address: str
    sip_server_user: str
    sip_server_password: str
    sip_server_timeout: int
    sip_address_type: SIPAddressType
    public_address: str
    stun_address: str
    stun_request_period: int


@dataclass
class CommandProdysV4Configuration:
    line_1_port: int
    enable_control_port: bool
    auto_answer: bool
    enable_fec: bool
    audio_packets_per_fec_packet: int
    fec_delay: int


@dataclass
class CommandProdysV4ConfigurationResponse:
    line_1_port: int
    enable_control_port: bool
    auto_answer: bool
    enable_fec: bool
    audio_packets_per_fec_packet: int
    fec_delay: int


@dataclass
class CommandStreamingTXCodecGetConfiguration:
    codec: Codec
    audio_algorithm: AudioAlgorithm


@dataclass
class CommandStreamingTXCodecGetConfigurationResponse:
    valid_request: bool
    time_between_packets: int


@dataclass
class CommandStreamingTXCodecSetConfiguration:
    codec: Codec
    audio_algorithm = AudioAlgorithm
    time_between_packets: int


@dataclass
class CommandStreamingRXCodecGetConfiguration:
    codec = Codec


@dataclass
class CommandStreamingRXCodecGetConfigurationResponse:
    valid_request: bool
    auto_adjustment: bool
    auto_min_delay: bool
    auto_min_delay_duration: int
    auto_max_delay: bool
    auto_max_delay_duration: int
    manual_max_delay: int


@dataclass
class CommandStreamingRXCodecSetConfiguration:
    codec: Codec
    auto_adjustment: bool
    auto_min_delay: bool
    auto_min_delay_duration: int
    auto_max_delay: bool
    auto_max_delay_duration: int
    manual_max_delay: int


@dataclass
class CommandGetStreamingStatsV2:
    codec: Codec


@dataclass
class CommandGetStreamingStatsV2Response:
    valid_request: bool
    call_in_progress: bool
    detected_bitrate: int
    rx_buffer_last_value: int
    lost_packets: int
    disordered_packets: int
    recovered_by_fec_packets: int
    current_jitter: int
    max_jitter: int
    mean_jitter: int


@dataclass
class CommandEthernetGetSpeed:
    ethernet_interface: EthernetInterface


@dataclass
class CommandEthernetGetSpeedResponse:
    valid_request: bool
    interface_enabled: bool
    ethernet_negotiated_speed: EthernetNegotiatedSpeed


@dataclass
class CommandStreamingStatsReset:
    codec: Codec


@dataclass
class CommandAudioModeConfigPCMV2:
    codec: Codec
    bits_sample: BitsSample
    audio_mode: AudioMode
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandAudioModeConfigG711V3:
    codec: Codec
    g711_law: G711Law
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandAudioModeConfigG722V3:
    codec: Codec
    encoder_mix: EncoderMix
    aux_data: bool
    g722_dither: bool


@dataclass
class CommandAudioModeConfigAPTXV2:
    codec: Codec
    aptx_type: APTXType
    bit_rate: BitRate
    audio_mode: AudioMode
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandGetAudioModeAlgorithm:
    codec: Codec


@dataclass
class CommandEncoderGetAudioModeAlgorithmResponse:
    valid_request: bool
    audio_mode_algorithm: AudioModeAlgorithm


@dataclass
class CommandDecoderGetAudioModeAlgorithmResponse:
    valid_request: bool
    is_framed: bool
    audio_mode_algorithm: AudioModeAlgorithm


@dataclass
class CommandEncoderGetAudioModeAutoResponse:
    codec: Codec
    aux_data: bool


@dataclass
class CommandEncoderGetAudioModePCMResponse:
    codec: Codec
    bits_sample: BitsSample
    audio_mode: AudioMode
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandEncoderGetAudioModeG711Response:
    codec: Codec
    g711_law: G711Law
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandEncoderGetAudioModeG722Response:
    codec: Codec
    encoder_mix: EncoderMix
    aux_data: bool
    g722_dither: bool


@dataclass
class CommandEncoderGetAudioModeMPEGResponse:
    codec: Codec
    bit_rate: BitRate
    audio_mode: AudioMode
    mpeg_layer: MPEGLayer
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix
    bonding_type: BondingType
    mpeg_l3_polarity: MPEGL3Polarity


@dataclass
class CommandEncoderGetAudioModeAACResponse:
    codec: Codec
    bit_rate: BitRate
    audio_mode: AudioMode
    aac_mode: AACMode
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix
    bonding_type: BondingType


@dataclass
class CommandEncoderGetAudioModeAPTXResponse:
    codec: Codec
    aptx_type: APTXType
    bit_rate: BitRate
    audio_mode: AudioMode
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandDecoderGetAudioModePCMResponse:
    codec: Codec
    bits_sample: BitsSample
    audio_mode: AudioMode
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandDecoderGetAudioModeG711Response:
    codec: Codec
    g711_law: G711Law
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandDecoderGetAudioModeG722Response:
    codec: Codec
    encoder_mix: EncoderMix
    aux_data: bool
    g722_dither: bool


@dataclass
class CommandDecoderGetAudioModeMPEGResponse:
    codec: Codec
    bit_rate: BitRate
    audio_mode: AudioMode
    mpeg_layer: MPEGLayer
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix
    bonding_type: BondingType
    mpeg_l3_polarity: MPEGL3Polarity


@dataclass
class CommandDecoderGetAudioModeAACResponse:
    codec: Codec
    bit_rate: BitRate
    audio_mode: AudioMode
    aac_mode: AACMode
    frequency: Frequency
    crc: bool
    aux_data: bool
    encoder_mix: EncoderMix
    bonding_type: BondingType


@dataclass
class CommandDecoderGetAudioModeAPTXResponse:
    codec: Codec
    aptx_type: APTXType
    bit_rate: BitRate
    audio_mode: AudioMode
    encoder_mix: EncoderMix
    aux_data: bool


@dataclass
class CommandAudioModeOPUS:
    codec: Codec
    bit_rate: BitRate
    encoder_mix: EncoderMix
    aux_data: bool



from enum import IntEnum


class Command(IntEnum):
    ALIVE = 0
    CONNECT = 1
    COMMAND_RESPONSE = 3

    STATUS_DEVICE_NET_CHANGED = 10
    STATUS_LINE_STATUS_CHANGED = 11
    STATUS_ALARMS_STATUS_CHANGED = 12
    STATUS_DECODER_AUDIO_MODE_CHANGED = 13
    STATUS_ENCODER_AUDIO_MODE_CHANGED = 14

    COMMAND_GET_DEVICE_NET = 100
    COMMAND_SET_DEVICE_NET = 101
    COMMAND_CALL = 102
    COMMAND_HANG_UP = 103
    COMMAND_GET_LINE_STATUS = 104
    COMMAND_GET_VU_METERS = 105
    COMMAND_GET_MONITORS = 106
    COMMAND_ALARMS_GET_STATUS = 107
    COMMAND_DECODER_GET_AUDIO_MODE = 108
    COMMAND_ENCODER_GET_AUDIO_MODE = 109
    COMMAND_ENCODER_SET_AUDIO_MODE_PCM = 110
    COMMAND_ENCODER_SET_AUDIO_MODE_MPEG = 111
    COMMAND_ENCODER_SET_AUDIO_MODE_AAC = 112
    COMMAND_ENCODER_SET_AUDIO_MODE_G711 = 113
    COMMAND_ENCODER_SET_AUDIO_MODE_G722 = 114
    COMMAND_ENCODER_SET_AUDIO_MODE_AUTO = 115
    COMMAND_ENCODER_SET_AUDIO_MODE_APTX = 116
    COMMAND_GET_LINE_STATUS_DETAILS = 117
    COMMAND_ANSWER = 118
    COMMAND_ENCODER_SET_AUDIO_MODE_G711_V2 = 119
    COMMAND_ENCODER_SET_AUDIO_MODE_G722_V2 = 120
    COMMAND_ENCODER_SET_AUDIO_MODE_MPEG_V2 = 121
    COMMAND_IP_GET_STREAMING_PROTOCOL = 122
    COMMAND_IP_SET_STREAMING_PROTOCOL = 123
    COMMAND_X21_ENABLE_TX = 124
    COMMAND_LOAD_PRESET = 125
    COMMAND_CALL_FROM_BOOK = 126
    COMMAND_ALARMS_ACK = 127
    COMMAND_ALARMS_DELETE = 128
    COMMAND_CALL_FROM_BOOK_ALL_LINES = 129
    COMMAND_GET_DEVICE_NET_V2 = 130
    COMMAND_SET_DEVICE_NET_V2 = 131
    COMMAND_CALL_V2 = 132
    COMMAND_GET_LOADED_PRESET_INDEX = 133
    COMMAND_RUN_GPI_ACTION = 134
    COMMAND_ENABLE_NET_BACKUP = 135
    COMMAND_RESET_DEVICE = 136
    COMMAND_GET_LOADED_PRESET_NAME = 137
    COMMAND_GET_STREAMING_STATS = 138
    COMMAND_GET_SIP_SERVER_REGISTER_STATUS = 139
    COMMAND_GET_STUN_SERVER_REGISTER_STATUS = 140
    COMMAND_GET_CALL_DURATION = 141
    COMMAND_ALARMS_GET_LOG = 142
    COMMAND_AUDIO_GET_CONFIGURATION = 143
    COMMAND_AUDIO_SET_CONFIGURATION = 144
    COMMAND_SYS_GET_DEVICE_NAME = 145
    COMMAND_SYS_SET_DEVICE_NAME = 146
    COMMAND_SYS_BLINK_DEVICE = 147
    COMMAND_SYS_GET_VERSION_INFO = 148
    COMMAND_SIP_GET_CONFIGURATION = 149
    COMMAND_SIP_SET_CONFIGURATION = 150
    COMMAND_PRODYS_V4_GET_CONFIGURATION = 151
    COMMAND_PRODYS_V4_SET_CONFIGURATION = 152
    COMMAND_STREAMING_TX_GET_CODEC_CONFIGURATION = 153
    COMMAND_STREAMING_TX_SET_CODEC_CONFIGURATION = 154
    COMMAND_STREAMING_RX_GET_CODEC_CONFIGURATION = 155
    COMMAND_STREAMING_RX_SET_CODEC_CONFIGURATION = 156
    COMMAND_GET_STREAMING_STATS_V2 = 157
    COMMAND_ETHERNET_GET_SPEED = 158
    COMMAND_STREAMING_STATS_RESET = 159
    COMMAND_ENCODER_SET_AUDIO_MODE_PCM_V2 = 160
    COMMAND_ENCODER_SET_AUDIO_MODE_G711_V3 = 161
    COMMAND_ENCODER_SET_AUDIO_MODE_G722_V3 = 162
    COMMAND_ENCODER_SET_AUDIO_MODE_APTX_V2 = 163
    COMMAND_ENCODER_GET_AUDIO_MODE_ALGORITHM = 164
    COMMAND_DECODER_GET_AUDIO_MODE_ALGORITHM = 165
    COMMAND_ENCODER_GET_AUDIO_MODE_AUTO = 166
    COMMAND_ENCODER_GET_AUDIO_MODE_PCM = 167
    COMMAND_ENCODER_GET_AUDIO_MODE_G711 = 168
    COMMAND_ENCODER_GET_AUDIO_MODE_G722 = 169
    COMMAND_ENCODER_GET_AUDIO_MODE_MPEG = 170
    COMMAND_ENCODER_GET_AUDIO_MODE_AAC = 171
    COMMAND_ENCODER_GET_AUDIO_MODE_APTX = 172
    COMMAND_DECODER_GET_AUDIO_MODE_PCM = 173
    COMMAND_DECODER_GET_AUDIO_MODE_G711 = 174
    COMMAND_DECODER_GET_AUDIO_MODE_G722 = 175
    COMMAND_DECODER_GET_AUDIO_MODE_MPEG = 176
    COMMAND_DECODER_GET_AUDIO_MODE_AAC = 177
    COMMAND_DECODER_GET_AUDIO_MODE_APTX = 178
    COMMAND_ENCODER_SET_AUDIO_MODE_OPUS = 179
    COMMAND_ENCODER_GET_AUDIO_MODE_OPUS = 180
    COMMAND_DECODER_GET_AUDIO_MODE_OPUS = 181


class AudioMode(IntEnum):
    MONO = 1
    STEREO = 2
    JOINT = 3
    DUAL = 4


class BitsSample(IntEnum):
    BITS_SAMPLE_16 = 16
    BITS_SAMPLE_20 = 20
    BITS_SAMPLE_24 = 24


class EncoderMix(IntEnum):
    NONE = 0
    LR = 1
    LR_3_DB = 2
    LR_6_DB = 3


class IPCallType(IntEnum):
    UNICAST_BIDIRECTIONAL = 0
    UNICAST_UNIDIRECTIONAL_TX = 1
    MULTICAST_TX = 2
    MULTICAST_RX = 3
    UNICAST_UNIDIRECTIONAL_RX = 4


class LineStatus(IntEnum):
    NO_PHYSICAL_LINE = 0
    DISCONNECTED = 1
    DISCONNECTING = 2
    CALLING = 3
    RECEIVING_CALL = 4
    CONNECTED_CALLED = 5
    CONNECTED_RECEIVED = 6
    CALL_REJECTED = 7
    NOT_AVAILABLE = 8


class BitRate(IntEnum):
    BITRATE_FREE = 0
    BITRATE_8_KBPS = 1000
    BITRATE_16_KBPS = 2000
    BITRATE_24_KBPS = 3000
    BITRATE_32_KBPS = 4000
    BITRATE_40_KBPS = 5000
    BITRATE_48_KBPS = 6000
    BITRATE_56_KBPS = 7000
    BITRATE_64_KBPS = 8000
    BITRATE_72_KBPS = 9000
    BITRATE_80_KBPS = 10000
    BITRATE_88_KBPS = 11000
    BITRATE_96_KBPS = 12000
    BITRATE_104_KBPS = 13000
    BITRATE_112_KBPS = 14000
    BITRATE_120_KBPS = 15000
    BITRATE_128_KBPS = 16000
    BITRATE_144_KBPS = 18000
    BITRATE_160_KBPS = 20000
    BITRATE_168_KBPS = 21000
    BITRATE_192_KBPS = 24000
    BITRATE_224_KBPS = 28000
    BITRATE_240_KBPS = 30000
    BITRATE_256_KBPS = 32000
    BITRATE_280_KBPS = 35000
    BITRATE_288_KBPS = 36000
    BITRATE_320_KBPS = 40000
    BITRATE_336_KBPS = 42000
    BITRATE_384_KBPS = 48000
    BITRATE_480_KBPS = 60000
    BITRATE_576_KBPS = 72000


class MPEGLayer(IntEnum):
    L_NOT = 0
    L1 = 1
    L2 = 2
    L3 = 3


class Frequency(IntEnum):
    FREQ_8000 = 8000
    FREQ_11025 = 11025
    FREQ_12000 = 12000
    FREQ_16000 = 16000
    FREQ_22050 = 22050
    FREQ_24000 = 24000
    FREQ_32000 = 32000
    FREQ_44100 = 44100
    FREQ_48000 = 48000


class AACMode(IntEnum):
    MPEG2_LC = 5
    MPEG4_LC = 6
    MPEG4_ER_LD = 7
    MPEG4_HE = 8
    MPEG_ELD = 10


class APTXType(IntEnum):
    STANDARD = 0
    ENHANCED_16_BITS = 1
    ENHANCED_20_BITS = 2
    ENHANCED_24_BITS = 3


class BondingType(IntEnum):
    NONE = 0
    CCSTELOS = 1
    J52 = 2


class DeviceNet(IntEnum):
    IP = 0
    ISDN = 1
    X21 = 2
    NONE = 3
    PSTN = 4


class G711Law(IntEnum):
    A = 0
    MU = 1


class MPEGL3Polarity(IntEnum):
    NORMATIVE = 0
    INVERSE = 1


class StreamingProtocol(IntEnum):
    PRODYS = 0
    SIP = 1
    SAP = 2
    PRODYS_V2 = 3
    CCS = 4
    RTP = 5
    PRODYS_V3 = 6
    PRODYS_V4 = 7


class DeviceSubNet(IntEnum):
    DEVICE_SUBNET_NONE = 0
    DEVICE_SUBNET_ETHERNET = 1
    DEVICE_SUBNET_3G = 2


class ProntoNetCodecMode(IntEnum):
    SIMPLE = 0
    DOUBLE = 1


class Codec(IntEnum):
    CODEC_1 = 0
    CODEC_2 = 1
    CODEC_MAX = 2
    CODEC_NONE = 3


class AlarmStatus(IntEnum):
    NONE = 0
    ALARM_ACTIVE = 1
    ALARM_HISTORY = 2


class AudioConfig(IntEnum):
    ANALOG = 0
    ANALOG_EXTERNAL_SYNC = 1
    AESEBU_AUDIO_SYNC = 2
    AESEBU_EXTERNAL_SYNC = 3
    AESEBU_INTERNAL_SYNC = 4
    AESEBU_TRANSPARENT = 5
    AESEBU_UBIT_MASTER = 6
    AESEBU_UBIT_SLAVE = 7


class SIPAddressType(IntEnum):
    LOCAL = 0
    PUBLIC = 1
    STUN = 2


class AudioAlgorithm(IntEnum):
    PCM = 0
    G711 = 1
    G722 = 2
    MPEG_LAYER_2 = 3
    MPEG_LAYER_3 = 4
    MPEG2_AAC_LC = 5
    MPEG4_AAC_LC = 6
    MPEG4_AAC_LD = 7
    MPEG4_AAC_HE = 8
    MPEG4_AAC_ELD = 9
    APTX = 10
    APTX_ENH_16_BITS = 11
    APTX_ENH_20_BITS = 12
    APTX_ENH_24_BITS = 13


class EthernetInterface(IntEnum):
    LAN_1 = 0
    LAN_2 = 1


class EthernetNegotiatedSpeed(IntEnum):
    LINK_DOWN = 0
    SPEED_10_MB_HALF_DUPLEX = 1
    SPEED_10_MB_FULL_DUPLEX = 2
    SPEED_100_MB_HALF_DUPLEX = 3
    SPEED_100_MB_FULL_DUPLEX = 4


class AudioModeAlgorithm(IntEnum):
    PCM = 0
    MPEG = 1
    AAC = 2
    G711 = 3
    G722 = 4
    AUTO = 5
    APTX = 6


class Acknowledge(IntEnum):
    ERROR = 0
    SUCCESS = 1



from setuptools import setup


def readme_file_contents():
    with open('README.rst') as readme:
        data = readme.read()
    return data


setup(
    name='prontonet-py',
    version='0.0.1',
    description='Python SDK library for Prontonet Family v2.5.4',
    long_description=readme_file_contents(),
    author='avc',
    author_email='avcsec@protonmail.com',
    license='MIT',
    packages=['prontonet'],
    zip_safe=False,
    install_requires=[]
)
